/*
*** FFR by DG1BBQ@DB0CL.DEU.EU
***
*** range.c
***
 */

#include "ffr.h"
/*
***
*** evaluate specified range. form: '-5,1-2,5,9-' means:
*** entries 1,2,3,4,5 and 9-500. ranges do not need to be in correct order.
*** overlapping of ranges is allowed, but space characters are not.
***
 */
static char ignored[] = "Range ignored.\n";

void get_range (void)
{
  int i, n, flag, start, end;
  char *p;

  i = n = flag = 0;

  /* get first range */
  p = strtok (extract, ",");
  do
  {
    flag = FALSE;
    for (i = 0;i<(int)strlen(p);i++)
      if ((p[i] < '0' || p[i] > '9') && p[i] != '-')
      {
        printf ("\007Invalid characters in range: '%s'\n", p);
        puts (ignored);
        flag = TRUE;
      }
    if (flag)
      continue;

    start = end = 0;
    sscanf (p, "%d-%d", &start, &end);

    if (!end && p[(int)strlen(p)-1] == '-')
      end = 500;

    if (start < 0)
    {
      end = start * -1;
      start = 1;
    }
    if (!end)
       end = start;
    if (start == 0)
    {
      printf ("\007Start equal 0: '%s'. (Start > 0!)\n", p);
      puts (ignored);
      continue;
    }
    if (end < start)
    {
      printf ("\007End smaller than beginning: '%s'\n", p);
      puts (ignored);
      continue;
    }
    if (end > 500)
    {
      printf ("\007Range too big: '%s'. (Max 500!)\n", p);
      puts (ignored);
      continue;
    }
    /* add range to array */
    for (i = start;i <= end; i++)
      numbers[n++] = i;
    numbers[n] = 1000;
    /* sort */
    qsort (numbers, n, sizeof(int), (fcmp) comp_int);

    /* delete doubles */
    i = 1;
    n = 0;
    while (numbers[i] != 1000)
    {
      if (numbers[n] == numbers[i])
        i++;
      else
        numbers[++n] = numbers[i++];
    }
    numbers[++n] = 1000;
  }
  while((p = strtok (NULL, ",")) != NULL); /* get next range */

  *extract = EOS;

  if (numbers[0] == 1000)
  {
    printf ("\007No valid ranges specified.\n");
    exit (-1);
  }
}

/*
*** Read an index file and build a list of entry numbers of it.
*** An index file is a list generated by FFR by redirecting output
*** from stdout to a file while listing entries.
***
*** Example: ffr savefile > list
***
*** Entries in that list can be marked by putting an additional '#'
*** in front of the number of an entry.
***
*** Example:
*** ##1     SOFTWARE @ALLE   de:DG1BBQ 16.10.91 17:29  90   4000 Bytes
*** !       7+ Info:CRC in ERR/COR-Files
*** !
*** +--- The additional '#' marks the entry.
***
*** If e.g. the entries 1,5,24 and 70 were marked in the file, this
*** function will kill those entries if invoked in the following
*** manner:
***
*** ffr savefile -k * -gi list
***                 !
***                 +--- This is just a dummy.
***
*** This will also work for the -l and -x commands.
***
 */
void get_index (char *name)
{
  FILE *index;
  int i = -1;
  char string[MAXGET+1];

  if ((index = fopen (name, OPEN_READ_TEXT)) == NULL)
  {
    printf ("\007Can't open index file '%s'.\n", name);
    exit (0);
  }

  while (fgets (string, MAXGET, index) != NULL)
  {
    if (!strncmp (string, "##", 2))
    {
      if (sscanf (string+2, "%d", &numbers[++i]) != 1)
        i--;
    }
    if (i == 999)
      break;
  }
  numbers[++i] = 1000;

  fclose (index);

  if (!i)
  {
    printf ("\007No marked entries found in '%s'.\n", name);
    exit (0);
  }
}